@@ -15,13 +15,6 @@ sub new {
     return $self;
 }
 
-# Add a scorer for a sub-query of the BooleanQuery.
-sub add_subscorer {
-    my ( $self, $subscorer, $occur ) = @_;
-    push @{ $self->_get_subscorer_storage }, $subscorer;
-    $self->_add_subscorer( $subscorer, $occur );
-}
-
 1;
 
 __END__
@@ -31,29 +24,43 @@ __XS__
 MODULE = KinoSearch    PACKAGE = KinoSearch::Search::BooleanScorer
 
 void
-_init_child(obj)
-    Scorer *obj;
+_init_child(scorer)
+    Scorer *scorer;
 PPCODE:
-    Kino_BoolScorer_init_child(obj);
+    Kino_BoolScorer_init_child(scorer);
+
+=for comment
+Add a scorer for a sub-query of the BooleanQuery.
+
+=cut
 
 void 
-_add_subscorer(obj, subscorer, occur)
-    Scorer *obj;
-    Scorer *subscorer;
+add_subscorer(scorer, subscorer_sv, occur)
+    Scorer *scorer;
+    SV     *subscorer_sv;
     char   *occur;
+PREINIT:
+    BoolScorerChild* child;
+    Scorer *subscorer;
+    SV     *subscorer_sv_copy;
 PPCODE:
-    Kino_BoolScorer_add_subscorer(obj, subscorer, occur);
+    child = (BoolScorerChild*)scorer->child;
+    Kino_extract_struct(subscorer_sv, subscorer, 
+        Scorer*, "KinoSearch::Search::Scorer");
+    subscorer_sv_copy = newSVsv(subscorer_sv);
+    av_push(child->subscorers_av, subscorer_sv_copy);
+    Kino_BoolScorer_add_subscorer(scorer, subscorer, occur);
 
 SV*
-_boolean_scorer_set_or_get(obj, ...)
-    Scorer* obj;
+_boolean_scorer_set_or_get(scorer, ...)
+    Scorer* scorer;
 ALIAS:
     _get_subscorer_storage = 2
 PREINIT:
     BoolScorerChild* child;
 CODE:
 {
-    child = (BoolScorerChild*)obj->child;
+    child = (BoolScorerChild*)scorer->child;
 
     /* if called as a setter, make sure the extra arg is there */
     if (ix % 2 == 1 && items != 2)
@@ -68,10 +75,10 @@ CODE:
 OUTPUT: RETVAL
 
 void
-DESTROY(obj)
-    Scorer *obj;
+DESTROY(scorer)
+    Scorer *scorer;
 PPCODE:
-    Kino_BoolScorer_destroy(obj);
+    Kino_BoolScorer_destroy(scorer);
 
 __H__
 
@@ -204,7 +211,6 @@ void
 Kino_BoolScorer_add_subscorer(Scorer* main_scorer, Scorer* subscorer, 
                               char *occur) {
     BoolScorerChild *child;
-    U32              mask;
     BoolSubScorer   *bool_subscorer;
 
     child = (BoolScorerChild*)main_scorer->child;
@@ -311,11 +317,10 @@ Kino_BoolScorer_next(Scorer* scorer) {
 
 float
 Kino_BoolScorer_score(Scorer* scorer){
-    BoolScorerChild *child;
-    MatchBatch      *mbatch;
+    BoolScorerChild *child = (BoolScorerChild*)scorer->child;
+    MatchBatch      *mbatch = child->mbatch;
     U32              masked_doc;
-    child = (BoolScorerChild*)scorer->child;
-    mbatch = child->mbatch;
+    float            score;
 
     if (child->coord_factors == NULL) {
         Kino_BoolScorer_compute_coord_factors(scorer);
@@ -323,7 +328,7 @@ Kino_BoolScorer_score(Scorer* scorer){
 
     /* retrieve the docs accumulated score from the MatchBatch */
     masked_doc = child->doc & KINO_MATCH_BATCH_DOC_MASK;
-    float score = mbatch->scores[masked_doc];
+    score = mbatch->scores[masked_doc];
 
     /* assign bonus for multi-subscorer matches */
     score *= child->coord_factors[ mbatch->matcher_counts[masked_doc] ];
@@ -384,7 +389,7 @@ Copyright 2005-2006 Marvin Humphrey
 
 =head1 LICENSE, DISCLAIMER, BUGS, etc.
 
-See L<KinoSearch|KinoSearch> version 0.08.
+See L<KinoSearch|KinoSearch> version 0.09.
 
 =end devdocs
 =cut
